       program  plamv                                                  
! * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *           
!        Resonance method for scattered operator                        
!         for an elastic half space                                     
!                                                                       
!          Formulated and Coded  By Prof. T. Touhei                     
!         **identification of music  point-like scatters**              
!                                                                       
!          For the basis of ker F^{*}                                   
!              -> use of left eigen vector of F whose                   
!                 eigenvaluse are less than eps                         
!                 eigenvalue analysis is due to the use of zgeev        
!                 bandled in Lapack                                     
!                                                                       
!                                                                       
!          Branch of \nu and \gamma in propa are corrected according to 
!           the steepest descent path method                            
!             -i \gamma  &  -i \nu                                      
!                                                                       
!                                                                       
!          Note : use of infc1  on Mar. 13 2017                         
!                       =0    use of direct wavenumber intg for F       
!                       =1                                              
!                                                                       
!                                                                       
!                                                                       
!                                                                       
!          ver  Jan 11, 2018                                             
!          Intensive check and test were carried out during Jan to Feb 2018
!          The test was for greensda and proj
!          see /plam/log
!          
!          ver  Feb. 19,2018
!          Intesive check and test were also keep carring out during
!           february and march.  Several bugs were thought to be removed
!          on mar. 2                                                                       
!
!
!         ver March 2, 2018   
!         ver March 23, 2018  check use of zgeev (see ref note on mar. 24)
!                             consider Paraview output
!         ver May 2,    2018  employ test function as the dipole directivity
! 
!
!         ver May 10,  2018   scatp for \mu  
!               is corrected (see memo may10)
!          ver May11, 2018     noise   is corrected
! * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *           
!                                                                        
!
      implicit  real*8 (a-h,o-y)                                        
      implicit  complex*16(z)                                           
!                                                                       
!                                                                       
      include 'mpif.h'                                                  
      integer istatus(MPI_STATUS_SIZE)                                  
!                                                                       
!                                                                       
!                                                                       
!                                                                       
       parameter  (np=500)                                              
       parameter  (nf=np)                                               
       parameter  (nf3=np*3)                                            
       parameter  (ne=2000)                                             
       parameter  (ne3=ne*3)  
       parameter  (ne6=ne*6)                                          
       parameter  (ns=40000)                                            
       parameter  (ns3=ns*3)                                            
       parameter  (nmax=nf3)                                            
       parameter  (nf3nf3=nf3*nf3)
       parameter  (nbf=10)                                       
!                                                                       
       parameter  (lwork=5*nmax)                                        
       parameter  (liwork=2*nmax)                                     
!                                                                       
!                                                                       
       common    /media/ro,ct,cl,rmu,ray,rtl,rlam                       
       common    /dynam/omega,dka,dkb,eps                               
       common    /slgrd/xlgrd(ns,3),dxsl1,dxsl2,dxsl3,nns1,nns2,nns3, &
     &                 ipole       
       common    /fsgrd/xfgrd(nf,3),nnf,nne 
       common    /tesct/tlam(nbf),tmu(nbf),trho(nbf),nnbf
       common    /cutof/epst                                            
       common    /inpar/divk,deltk,rmax,fdivk,fdeltk,infc1              
       common    /fagrd/xfang(nf,2),xcont(3)                            
       common    /pjvct/zp(nf3),zps(nf3),zpst(nf3),zdapr(nf)            
       common    /sjvct/zs(nf3),zss(nf3),zsst(nf3),zt(nf3), &        
     &                 zdbpr(nf),zdtpr(nf)                             
       common    /panav/iawave                                          
       common    /myrak/ myrank,nprocs                                  
       common    /mprak/ ista1,iend1,ista2,iend2,  &
                         & ista3,iend3,ista4,iend4
       common    /anois/ mdran,rlevel                                   
       common    /scatv/ xegrd(ne,3),flam(ne),fmu(ne),fro(ne),dxol(ne),&
     &                   mvolm(ne)
!
!
!
!
!
!                                                                       
!                                                                       
       dimension    distvt(ns),distvtm(ns),distvmp(ns)
!                                                                       
       real*8, allocatable :: distv(:)                                  
!                                                                       
       complex*16, allocatable :: zwkscv01(:)                        
       complex*16, allocatable :: zwkscv02(:)                       
       complex*16, allocatable :: zwkscv03(:)
       complex*16, allocatable :: zwkscv04(:)                        
       complex*16, allocatable :: zwkscv05(:)                       
       complex*16, allocatable :: zwkscv06(:)
       complex*16, allocatable :: zwkscv07(:)                       
       complex*16, allocatable :: zwkscv08(:)                       
       complex*16, allocatable :: zwkscv09(:)
       complex*16, allocatable :: zwkscv10(:)                         
       complex*16, allocatable :: zwkscv11(:)                         
       complex*16, allocatable :: zwkscv12(:)                        
!
       integer,    allocatable :: seed(:)                               
                                                                        
!                                                                       
       character*20    outpt(5)                                         
!                                                                       
       integer     seedsize                                             
       dimension   yrand(nf3nf3,2)                                    
!                                                                       
       dimension  xf(3),yf(3)                                 
       dimension  zwksc(nf3,ne3,4) 
       dimension  zhfrv(nf3,ne),zhfrs(nf3,ne6),zhfrg(nf3,ne3)
       dimension  zopea(nf3,nf3),zopeaf(nf3,nf3)
!            
!
       dimension  zwkscv01t(ne3),zwkscv02t(ne3),zwkscv03t(ne3)        
       dimension  zwkscv04t(ne3),zwkscv05t(ne3),zwkscv06t(ne3)       
       dimension  zwkscv07t(ne3),zwkscv08t(ne3),zwkscv09t(ne3)         
       dimension  zwkscv10t(ne3),zwkscv11t(ne3),zwkscv12t(ne3)         
!                                                                       
!                                                                       
       dimension  zrzw(nf3), zwkvl(nf3,nf3),zwkvr(nf3,nf3)              
       dimension  zwork(lwork),rwork(liwork)                            
       dimension  zwksa01(nf3),zwksa02(nf3),zwksa03(nf3)
! 
!   
       dimension  zhpf(nf3,3)                           
!                                                                       
       dimension  zalproj(nf,3,3),zbbproj(nf,3,3),zbtproj(nf,3,3)       
       dimension  zwkprojl(nf,3,3),zwkprojr(nf,3,3)                     
!                                                                       
!                                                                       
      dimension   idisp1(0:40),ircnt1(0:40),ierrv(40)                  
!                                                                       
!                                                                       
!                                                                       
       call  mpi_init(ierr)                                             
       call  mpi_comm_size(MPI_COMM_WORLD,nprocs, ierr )                
       call  mpi_comm_rank(MPI_COMM_WORLD,myrank,ierr)                  
       call random_seed(size=seedsize)       
       allocate(seed(seedsize))                                         
!                                                                       
!                                                                       
!                                                                       
!                                                                       
       if(myrank.eq.0)then                                              
       call    input(outpt)                                             
       call    rayle(rtl,ray)                                           
       endif                                                            
!                                                                       
      call mpi_bcast(ro,7,mpi_real8,0,MPI_COMM_WORLD,ierrv(1))          
      call mpi_bcast(omega,4,mpi_real8,0,MPI_COMM_WORLD,ierrv(2))       
      call mpi_bcast(divk,5,mpi_real8,0,MPI_COMM_WORLD,ierrv(3))        
      call mpi_bcast(infc1,1,mpi_integer,0,MPI_COMM_WORLD,ierrv(4))     
      call mpi_bcast(epst,1,mpi_real8,0,MPI_COMM_WORLD,ierrv(5))        
      call mpi_bcast(nnf,1,mpi_integer,0,MPI_COMM_WORLD,ierrv(6))       
      call mpi_bcast(nne,1,mpi_integer,0,MPI_COMM_WORLD,ierrv(7))       
      call mpi_bcast(nns1,1,mpi_integer,0,MPI_COMM_WORLD,ierrv(8))      
      call mpi_bcast(nns2,1,mpi_integer,0,MPI_COMM_WORLD,ierrv(9))      
      call mpi_bcast(nns3,1,mpi_integer,0,MPI_COMM_WORLD,ierrv(10))     
      call mpi_bcast(ipole,1,mpi_integer,0,MPI_COMM_WORLD,ierrvt)     
      call mpi_bcast(dxsl1,1,mpi_real8,0,MPI_COMM_WORLD,ierrvt)     
      call mpi_bcast(dxsl2,1,mpi_real8,0,MPI_COMM_WORLD,ierrvt)     
      call mpi_bcast(dxsl3,1,mpi_real8,0,MPI_COMM_WORLD,ierrvt)     
      call mpi_bcast(iawave,1,mpi_integer,0,MPI_COMM_WORLD,ierrv(11))   
      call  mpi_bcast(xfgrd(1,1),nf3,mpi_real8,0,MPI_COMM_WORLD, &      
     &                ierrv(12))                                        
      call mpi_bcast(xlgrd(1,1),ns3,mpi_real8,0,MPI_COMM_WORLD, &       
     &               ierrv(13))                                         
      call mpi_bcast(mdran,1,mpi_integer,0,MPI_COMM_WORLD,ierrv(14))    
      call mpi_bcast(rlevel,1,mpi_real8,0,MPI_COMM_WORLD,ierrv(15))     
!
      call mpi_bcast(xegrd(1,1),ne3,mpi_real8,0,MPI_COMM_WORLD, &       
     &               ierrv(16))                                         
      call mpi_bcast(flam(1),ne ,mpi_real8,0,MPI_COMM_WORLD, &         
     &               ierrv(17))                                         
      call mpi_bcast(fmu(1),ne ,mpi_real8,0,MPI_COMM_WORLD, &         
     &               ierrv(18))                                         
      call mpi_bcast(fro(1),ne ,mpi_real8,0,MPI_COMM_WORLD, &         
     &               ierrv(19))                                         

!
!                                                                       
       ierr=0                                                           
       do  kk=1, 18                                                     
       ierr=ierr+ierrv(kk)                                              
       enddo                                                            
!                                                                       
       if(ierr.ne.0) write(*,*)' problem in bcst err=',ierr             
!                                                                       
!                                                                       
       nns=nns1*nns2*nns3
       nne3=nne*3
       nne6=nne*6                                               
       nnf3=nnf*3 
!                                                     
       call  para_range(1,nns,nprocs,myrank,ista1,iend1)                
       call  para_range(1,nne,nprocs,myrank,ista2,iend2)                
       call  para_range(1,nne3,nprocs,myrank,ista3,iend3)              
       call  para_range(1,nne6,nprocs,myrank,ista4,iend4)             
!                                                                       
       allocate ( distv(ista1:iend1), stat=ierr)                        
!
       allocate (zwkscv01(3*(ista2-1):3*iend2), stat=ierr)              
       allocate (zwkscv02(3*(ista2-1):3*iend2), stat=ierr)              
       allocate (zwkscv03(3*(ista2-1):3*iend2), stat=ierr)              
       allocate (zwkscv04(3*(ista2-1):3*iend2), stat=ierr)              
       allocate (zwkscv05(3*(ista2-1):3*iend2), stat=ierr)              
       allocate (zwkscv06(3*(ista2-1):3*iend2), stat=ierr)              
       allocate (zwkscv07(3*(ista2-1):3*iend2), stat=ierr)              
       allocate (zwkscv08(3*(ista2-1):3*iend2), stat=ierr)              
       allocate (zwkscv09(3*(ista2-1):3*iend2), stat=ierr)              
       allocate (zwkscv10(3*(ista2-1):3*iend2), stat=ierr)              
       allocate (zwkscv11(3*(ista2-1):3*iend2), stat=ierr)              
       allocate (zwkscv12(3*(ista2-1):3*iend2), stat=ierr)              
!                                                                       
!                                                                       
       idisp1(myrank)=ista1-1                                           
       ircnt1(myrank)=iend1-ista1+1                                     
!                                                                       
        do ii=0,nprocs-1                                                
         call mpi_bcast(idisp1(ii),1,mpi_integer,ii, &                 
     &                       MPI_COMM_WORLD,ierr)                      
         call mpi_bcast(ircnt1(ii),1,mpi_integer,ii, &                
     &                       MPI_COMM_WORLD,ierr)                       
        enddo                                                           
                                                                        
!                                                                       
         iist1=idisp1(myrank)+1                                         
         iict1=ircnt1(myrank)                                           
!                                                                       
!                                                                       
!                                                                       
!        goto 8000
!                                                                       
!                                                                       
        call  scatp(zwkscv01t,zwkscv02t,zwkscv03t,zwkscv04t, &
      &  zwkscv05t,zwkscv06t,zwkscv07t, zwkscv08t,zwkscv09t,zwkscv10t,&
      &  zwkscv11t, zwkscv12t,zwkscv01,zwkscv02,zwkscv03,zwkscv04, &
      &  zwkscv05,zwkscv06,zwkscv07,zwkscv08,zwkscv09,zwkscv10, &
      &  zwkscv11,zwkscv12,zwksc,zhfrv,zhfrs,zhfrg,zopea)   
! 
! 
                       
        call mpi_barrier(MPI_COMM_WORLD,ierr)                           
!                                                                       
! 
!                                                       
!                                                                       
       call random_seed(get=seed)                                       
!                                                                       
        if(mdran.eq.1) then                                             
        nnfnnf=nnf*nnf*9                                                
        do  it=1, 2                                                     
          do i = 1, nnfnnf                                              
          call random_number(xran)                                      
          yrand(i,it)=xran                                              
          enddo                                                         
       enddo                                                            
       call  noise(yrand,zopea,rlevel)                                  
       if(myrank.eq.0)    write(*,*)' end of random number generate'    
!                                                                       
       if(myrank.eq.0) then                                             
          open(unit=2010,file='random-1')                               
          open(unit=2011,file='random-2')                               
          open(unit=2012,file='ran-sct')                                
!                                                                       
          do  ii=1, nnfnne                                              
          write(2010,*) ii,yrand(ii,1)                                  
          write(2011,*) ii,yrand(ii,2)                                  
          enddo                                                         
!                                               
          close(unit=2010)                                              
          close(unit=2011)                                              
!                                                                       
          do ii=1, nnf3                                                 
           do  jj=1, nnf3                                               
           write(2012,*)ii,jj,cdabs(zopea(ii,jj))                       
           enddo                                                        
           write(2012,*)'     '                                         
          enddo                                                         
         close(unit=2012)                                               
!                                                
       endif                                                            
        call mpi_barrier(MPI_COMM_WORLD,ierr)                           
!                                           
      endif                                                             
!          goto 8000                                                    
!                                                                       
       do  kk=1, nns
       distvmp(kk)=1.0d0
       enddo
!                                                                       
!                                                                       
!                                                                       
!                                                                       
!      %%%% kpole-loop %%%%                                                                
       do  kpole=1, 4 
!                                                                       
!                                                                       
          do   kk=1, nns        
           distvtm(kk)=1.0d0          
          enddo                     
!                                                                       
!                                   
!          %%%%% kalpha-loop %%%%      
           do   kalpha=1, 3            
!                                                                       
            if(kalpha.eq.1)iawave=11                
            if(kalpha.eq.2)iawave=22               
            if(kalpha.eq.3)iawave=33              
!
!            %%%%%  isa-loop  %%%%%   
             do   isa=ista1, iend1                                     
!                                                                       
              xcont(1)=xlgrd(isa,1)          
              xcont(2)=xlgrd(isa,2)         
              xcont(3)=xlgrd(isa,3)        
!                                                                       
! 
              call   proj(zalproj,zbbproj,zbtproj) 
! 
! 
              call   opera(zopea,zopeaf, zwkvl, zwkvr, zwork,rwork,&
      &                 zalproj,zbbproj,zbtproj,zwkprojl, zwkprojr,&
      &                 iawave)  
! 
              call  sample(zwkvl,zhpf,zwksa01,zwksa02,zwksa03, &
      &                   distvel,iawave,kpole)
!                                                                       
              distv(isa)=distvel  
             enddo          
!            %%%  isa-loop %%%  
             if(myrank.eq.0) write(*,*)' kpole and kalpha=',kpole,kalpha
             call  mpi_barrier(MPI_COMM_WORLD,ierr)                   
             call  mpi_allgatherv(distv(iist1),iict1,mpi_real8, &      
       &        distvt,ircnt1,idisp1,mpi_real8,MPI_COMM_WORLD,ierr)    
!                                                                       
!                                                                       
           do  inns=1, nns                                         
                  distvtm(inns)=distvt(inns)*distvtm(inns)              
           enddo                                                   
!                                                                       
         enddo                                                     
!        %%%%% kalpha-loop%%%%                                                               
!                                                                       
!                                                                       
!         %%%%%%%%% LIST WITH Kpole %%%%%%%    
          if(myrank.eq.0) then                                          
           iun=100+kpole                            
           open(unit=iun,file=outpt(kpole))         
           write(iun,*)dxsl1,dxsl2,dxsl3
           nns12=nns1*nns2 
            do  kk=1, nns3 
             do  jj=1, nns2                                          
              do   ii=1, nns1         
               kkp=(kk-1)*nns12+(jj-1)*nns1+ii      
               yf(1)=xlgrd(kkp,1)  
               yf(2)=xlgrd(kkp,2)
               yf(3)=xlgrd(kkp,3) 
               write(iun,1239) yf(1),yf(2), yf(3),distvtm(kkp)    
              enddo                                                   
             enddo                
            enddo                     
            write(*,*)'** end of kpole loop  output map=',outpt(kpole)
            close(unit=iun)                                         
          endif                                                        
!         %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 1239           format(3f10.5, e15.7) 
!                                                                       
!                                                                       
!                                                                       
           do  inns=1, nns                                         
                  distvmp(inns)=distvtm(inns)*distvmp(inns)     
           enddo
!  
      enddo
!     %%%%%% Kpole loop %%%%%%%%
! 
! 
!                                                                       
!                                                                       
!         %%%%%%%%% LIST WITH Kpole %%%%%%%    
          if(myrank.eq.0) then                                          
           iun=105                          
           open(unit=iun,file=outpt(5))         
           write(iun,*)dxsl1,dxsl2,dxsl3
           nns12=nns1*nns2 
            do  kk=1, nns3 
             do  jj=1, nns2                                          
              do   ii=1, nns1         
               kkp=(kk-1)*nns12+(jj-1)*nns1+ii      
               yf(1)=xlgrd(kkp,1)  
               yf(2)=xlgrd(kkp,2)
               yf(3)=xlgrd(kkp,3) 
               write(iun,1239) yf(1),yf(2), yf(3),distvmp(kkp)    
              enddo                                                   
             enddo                
            enddo                     
            write(*,*)'** end of kpole loop  output map=',outpt(5)
            close(unit=iun)                                         
          endif                                                        
!         %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
!                                                                       
!                                                                       
!                                                                       
 8000 call  mpi_finalize(ierr)                                          
       if(ierr.ne.0) write(*,*)' final ierr=',ierr                      
!                                                                       
!                                                                       
       stop                                                             
       end                                                              
!                                                                       
!                                                                       
! * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *       
      subroutine para_range(n1,n2,nprocs,irank,ista,iend)               
! * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *       
!                                                                       
       iwork1=(n2-n1+1)/nprocs                                          
       iwork2=mod(n2-n1+1,nprocs)                                       
       ista=irank*iwork1+n1+min(irank,iwork2)                           
       iend=ista+iwork1-1                                               
       if(iwork2 > irank ) iend=iend+1                                  
       end                                                              
!                                                                       
!                                                                       
!                                                                       
! * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
      subroutine   input(outpt)                                         
! * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
!
!                                                                       
      implicit real*8 (a-h,o-y)   
      implicit integer(i-n)                                      
      implicit complex*16 (z)                                           
!                                                                       
!                                                                       
       parameter  (np=500)                                              
       parameter  (nf=np)                                               
       parameter  (nf3=np*3)                                            
       parameter  (ne=2000)                                             
       parameter  (ne3=ne*3)                                            
       parameter  (ns=40000)              
       parameter  (nbf=10)                              
!                                                                       
!                                                                       
       character*6    com,dummy                                         
       character*20   point,mapname,outpt(5),dsn,dsnf(nbf)  
!                                                                        
       common    /media/ro,ct,cl,rmu,ray,rtl,rlam                       
       common    /dynam/omega,dka,dkb,eps   
       common    /fsgrd/xfgrd(nf,3),nnf,nne 
       common    /tesct/tlam(nbf),tmu(nbf),trho(nbf),nnbf     
       common    /fagrd/xfang(nf,2),xcont(3)                            
       common    /slgrd/xlgrd(ns,3),dxsl1,dxsl2,dxsl3,nns1,nns2,nns3,&
     &                  ipole
       common    /cutof/epst                                            
       common    /inpar/divk,deltk,rmax,fdivk,fdeltk,infc1              
       common    /panav/iawave                                          
       common    /anois/mdran,rlevel                                   
       common    /scatv/xegrd(ne,3),flam(ne),fmu(ne),fro(ne),dxol(ne),&
     &                  mvolm(ne)
!                                                                       
!                                                                       
!                                                                       
!                                                                       
       write(*,*)' input  dsn   '                                       
       read(*,1)dsn                                                     
!                                                                       
!                                                                       
      open(unit=10,file=dsn,status='unknown')                           
    5 read(10,2)com                                                     
    1 format(a20)                                                       
    2 format(a6)                                                        
!                                                                       
!       write(*,*)'com=',com                                            
      if(com.eq.'Bmedia') goto 1000                                     
      if(com.eq.'Dmedia') goto 1100                                     
      if(com.eq.'Wavein') goto 1200                                     
      if(com.eq.'Noise ') goto 1300                                     
      if(com.eq.'Fieldp') goto 1400                                     
      if(com.eq.'Files ') goto 1500                                     
      if(com.eq.'Sample') goto 1600                                     
      if(com.eq.'End   ') goto 1700                                     
      if(com.eq.'%    ') goto 5                                         
      goto 5                                                            
!                                                                       
 1000 read(10,1010)rlam,rmu,rho                                         
 1010 format(3f10.0)                                                    
!                                                                       
         rtl2=rmu/(rlam+2.0d0*rmu)                                      
         rtl= dsqrt(rtl2)                                               
!                                                                       
            goto 5                                                      
!                                                                       
!                                                                       
!                                                                       
 1100 read(10,1110) freq,eps,epst                                       
 1110 format(3f10.0)                                                    
!                                                                       
!                                                                       
        pi=3.141592653589793d0                                          
        omega=freq*2.0d0*pi                                             
        ct= dsqrt( rmu/rho)                                             
        cl=ct/rtl                                                       
        dkb=omega/ct                                                    
        dka=dkb*rtl                                                     
!                                                                       
!                                                                       
!                                                                       
            goto 5                                                      
!                                                                       
!                                                                       
 1200   continue                                                        
        read(10,1201) divk,deltk,rmax,fdivk,fdeltk,infc1                
 1201   format(5f10.0,i5)                                               
           goto 5                                                       
!                                                                       
 1300   continue                                                        
        read(10,1302)mdran, rlevel                                      
 1302   format(i5,f10.0)                                                
         goto 5                                                         
!                                                                       
!                                                                       
 1400 continue                                                          
      read(10,1402)nnf1,nnf2                                            
 1402 format(2i5)                                                       
      nnf=nnf1*nnf2                                                     
      read(10,1404)dxl1,dxl2,xsg1,xsg2                                  
 1403 format(2f10.0)                                                    
 1404 format(4f10.0)                                                    
!                                                                       
      dx1=dxl1/dfloat(nnf1-1)                                           
      dx2=dxl2/dfloat(nnf2-1)                                           
!                                                                       
       open(unit=22,file='chk-surface')                               
!      open(unit=23,file='chk-angle')                                   
      do  jj=1, nnf2                                                    
       do   ii=1, nnf1                                                  
       kk=(jj-1)*nnf1+ii                                                
       xfgrd(kk,1)= xsg1+dx1*dfloat(ii-1)                               
       xfgrd(kk,2)= xsg2+dx2*dfloat(jj-1)                               
       xfgrd(kk,3)= 0.0d0                                               
        write(22,*) xfgrd(kk,1),xfgrd(kk,2),xfgrd(kk,3)               
!                                                                       
        rdx1= xcont(1)-xfgrd(kk,1)                                      
        rdx2= xcont(2)-xfgrd(kk,2)                                      
        rdx3= xcont(3)-xfgrd(kk,3)                                      
        radius=dsqrt( rdx1*rdx1 +rdx2*rdx2 +rdx3*rdx3 )                 
        theta=dacos (rdx3/radius )                                      
        phi =datan2 (rdx2, rdx1 )                                       
        xfang(kk,1)=theta                                               
        xfang(kk,2)=phi                                                 
!                                                                       
!                                                                       
       enddo                                                            
        write(22,*)'    '                                            
      enddo                                                             
!                                                                       
!                                                                       
!                                                                       
       close(unit=22)                                               
!                                                                       
        goto 5                                                          
!                                                                       
!                                                                       
 1500  continue                                                         
!                                                                       
!                                                                       
       read(10,2) dummy                                                 
       read(10,1501)point                                               
 1501  format(a20)                                                      
       do kk=1, 5                                                       
       read(10,1501)outpt(kk)                                           
       enddo                                                            
!                                                                       
!                                                                       
!                                                                       
!                                                                       
!       write(*,*)'point=',point                                        
       open(unit=12,file=point, status='unknown')                       
       read(12,*)nne, nnbf    
 1502 format(2i5)                                                        
        write(*,*)' nne=',nne, ' nnbf =', nnbf 
       do  ii=1, nnbf
         read(12, 1503)tlam(ii),tmu(ii),trho(ii),dsnf(ii)
       enddo
! 
 1503  format(3f10.0,1x,a20)
!                                                                       
!                                                                       
       do ii=1, nnbf
       iun=24+ii-1
       open(unit=iun,file=dsnf(ii))  
!       write(*,*) 'blname=',dsnf(ii)
       enddo
!                                  
       do  ii=1, nne                                                    
!       write(*,*)' nne loop ii=',ii, nne                               
       read(12,*)xegrd(ii,1),xegrd(ii,2),xegrd(ii,3),dxol(ii),mvolm(ii) 
       xegrd(ii,3)=-xegrd(ii,3)
       xegrd(ii,1)= xegrd(ii,1)-7.0d0
       xegrd(ii,2)=-xegrd(ii,2)-7.0d0
!            
       ipx=mvolm(ii)
!       write(*,*)'ipx=',ipx
       dvolm=dxol(ii)**3
!       write(*,*)' dvolm=',dvolm
       flam(ii) =tlam( ipx )*dvolm 
       fmu(ii)  =tmu ( ipx )*dvolm
       fro(ii) =trho( ipx )*dvolm
!
        iun=24+ipx-1
        write(iun,*)xegrd(ii,1),xegrd(ii,2),xegrd(ii,3) 
! 
! 
       enddo                                                            
!                                                                       
       close(unit=12)                                                   
!
!
       do ii=1, nnbf
       iun=24+ii-1
       close(unit=iun)  
       enddo
!
!                                                                       
        goto 5                                                          
!                                                                       
!                                                                       
!                                                                       
!                                                                       
 1600 continue                                                          
       read(10,1601)dxs1,dxs2,dxs3,xst1,xst2,xst3                       
 1601  format(6f10.0)                                                   
       read(10,1602)nns1,nns2,nns3                             
 1602  format(3i5)                                                      
       read(10,1603)mapname                                             
 1603 format(a20)                                                       
!                                                                       
      open(unit=26,file=mapname) 
      open(unit=27,file='chk-psce-p')                                   
!                                                                       
      dx1=dxs1/dfloat(nns1-1)                                           
      dx2=dxs2/dfloat(nns2-1)  
      dx3=dxs3/dfloat(nns3)
!
      dxsl1=dx1
      dxsl2=dx2
      dxsl3=dx3
!
      nns12=nns1*nns2
       do  kk=1, nns3                                        
        do  jj=1, nns2       
         do   ii=1, nns1    
         kkp=(kk-1)*nns12+(jj-1)*nns1+ii    
         xlgrd(kkp,1)=xst1+dx1*dfloat(ii-1)                        
         xlgrd(kkp,2)=xst2+dx2*dfloat(jj-1)                       
         xlgrd(kkp,3)=xst3+dx3*dfloat(kk-1) 
         write(26,*)xlgrd(kkp,1),xlgrd(kkp,2),xlgrd(kkp,3)        
        enddo        
       enddo          
      enddo
!                                                                       
       do  ii=1, nne                                                    
       write(27,*)xegrd(ii,1),xegrd(ii,2),xegrd(ii,3)    
       enddo                                                            
!                                                                       
!                                                                       
       close(unit=26)                                                   
       close(unit=27)                                                   
        goto 5                                                          
!                                                                       
!                                                                       
!                                                                       
 1700 continue                                                          
!                                                                       
      close(unit=10)                                                    
!                                                                       
      write(*,*)'                                                     ' 
      write(*,*)'                                                     ' 
      write(*,*)'       ********************************************  ' 
      write(*,*)'       **                                        **  ' 
      write(*,*)'       **         Program plamv                  **  ' 
      write(*,*)'       **                                        **  ' 
      write(*,*)'       **         Construct far field Eq.        **  ' 
      write(*,*)'       **         from the scattered tensor      **  ' 
      write(*,*)'       **         for an elastic half-space      **  ' 
      write(*,*)'       **                                        **  ' 
      write(*,*)'       **     + mpi parallel version +           **  ' 
      write(*,*)'       ********************************************  ' 
      write(*,*)'                                                     ' 
      write(*,*)'            -- Ver. May  11, 2018--    ' 
!                                                                       
!                                                                       
      write(*,*)'     '                                                 
      write(*,*)'     '                                                 
      write(*,*)'  scatterer model=',point, ' nne=',nne                 
      write(*,*)'  n of grids for the tensor=',nnf       
      write(*,*)'  output map=',outpt(5)                                
      if(infc1.eq.0) write(*,*)'  direct Green function'     
      if(infc1.eq.1) write(*,*)'  far field Green function'
      write(*,*)'     '                                                 
!                                                                       
!       goto 9000
!                                                                       
!                                                                       
!                                                                       
 8000 return                                                            
 9000 stop                                                              
      end                                                               
!                                                                       
!                                                                       
!                                                                       
!                                                                       
! * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
      subroutine noise(yrand,zopea,rlevel)                              
! * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
!                                                                       
!                                                                       
!
      implicit real*8(a-h,o-y)                                          
      implicit complex*16(z)                                            
!                                                                       
      include 'mpif.h'                                                  
      integer istatus(MPI_STATUS_SIZE)                                  
!                                                                       
!                                                                       
!                                                                       
       parameter  (np=500)                                              
       parameter  (nf=np)                                               
       parameter  (nf3=np*3)                                            
       parameter  (ne=1000)                                             
       parameter  (ne3=ne*3)                                            
       parameter  (ns=40000)                                            
       parameter  (ns3=ns*3)                                            
       parameter  (nmax=nf3)                                            
       parameter  (nfnf3=nf3*nf3)                                       
!                                                                       
!                                                                       
       common    /media/ro,ct,cl,rmu,ray,rtl,rlam                       
       common    /dynam/omega,dka,dkb,eps 
       common    /fsgrd/xfgrd(nf,3),nnf,nne 
       common    /cutof/epst                                            
       common    /inpar/divk,deltk,rmax,fdivk,fdeltk,infc1              
       common    /panav/iawave                                          
       common    /myrak/ myrank,nprocs                                  
       common    /mprak/ ista1,iend1,ista2,iend2,  &
                       & ista3,iend3,ista4,iend4                      
!                                                                       
!                                                                       
       dimension   yrand(nfnf3,2),zopea(nf3,nf3)        
!                                                                       
!                                                                       
        pi=3.14159265358979323846d0                                     
        zi=(0.0d0,1.0d0)                                                
        zip2=zi*2.0d0*pi                                                
!                                                                       
!         write(*,*)' start of noise'                                   
!                                                                       
!                                                                       
!                                                                       
!                                                                       
!                                                                       
       nnf3=nnf*3                                                       
       rrnorm2=0.0d0                                                    
       rnnorm2=0.0d0                                                    
       do  ii=1, nnf3                                                   
        do   jj=1, nnf3                                                 
!                                                                       
        wwx= zopea(ii,jj)*dconjg(zopea(ii,jj))                          
        rrnorm2=rrnorm2+wwx                                             
!                                                                       
        itpara=(ii-1)*nnf3+jj                                           
        yran1=yrand(itpara,1)                                           
        yran2=yrand(itpara,2)                                           
        znois=yran1*cdexp(zipi2*yran2)                                  
        rnnorm2=rnnorm2+znois*dconjg(znois)                             
!                                                                       
        enddo                                                           
       enddo                                                            
!                                                                       
       rkap=rlevel*dsqrt(rrnorm2)/dsqrt(rnnorm2)                        
!                                                                       
       if(myrank.eq.0) write(*,*)' rkap=',rkap,rlevel                   
!                                                                       
!                                                                       
      do  ii=1, nnf3                                                    
        do   jj=1, nnf3                                                 
        itpara=(ii-1)*nnf3+jj                                           
        yran1=yrand(itpara,1)                                           
        yran2=yrand(itpara,2)                                           
        znois=yran1*cdexp(zipi2*yran2)                                  
        zopea(ii,jj)=rkap*znois+zopea(ii,jj)                            
       enddo                                                            
      enddo                                                            
!                                                                       
!                                                                       
!                                                                       
!                                                                       
!                                                                       
!                                                                       
!                                                                       
!                                                                       
!                                                                       
!                                                                       
         return                                                         
 9000    stop                                                           
         end                                                            
!                                                                       
!                                                                       
!                                                                       
!                                                                       
!                                                                       
!                                                                       
! * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
       subroutine  scatp(zwkscv01t,zwkscv02t,zwkscv03t, zwkscv04t,&
     & zwkscv05t,zwkscv06t,zwkscv07t, zwkscv08t, zwkscv09t, zwkscv10t,&
     & zwkscv11t, zwkscv12t, zwkscv01,zwkscv02,zwkscv03,zwkscv04,&
     & zwkscv05,zwkscv06,zwkscv07,zwkscv08,zwkscv09,zwkscv10,&
     & zwkscv11,zwkscv12,zwksc, zhfrv,zhfrs,zhfrg,zopea)           
! * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
!        
!                                                                        
       implicit  real*8(a-h,o-y)
       implicit  integer(i-n)                                        
       implicit  complex*16(z)                                          
!                                                                       
       include 'mpif.h'                                                 
       integer istatus(MPI_STATUS_SIZE)                                 
!                                                                       
       parameter  (np=500)                                              
       parameter  (nf=np)                                               
       parameter  (nf3=np*3)                                            
       parameter  (ne=2000)                                             
       parameter  (ne3=ne*3)
       parameter  (ne6=ne*6)                                            
       parameter  (nmax=nf3)                                            
       parameter  (lwork=5*nmax)                                        
       parameter  (liwork=2*nmax  )                                     
       parameter  (nbf=10)
!                                                                       
!                                                                       
      common    /media/ro,ct,cl,rmu,ray,rtl,rlam                       
      common    /dynam/omega,dka,dkb,eps                               
      common    /fsgrd/xfgrd(nf,3),nnf,nne         
      common    /cutof/epst                                            
      common    /inpar/divk,deltk,rmax,fdivk,fdeltk,infc1              
      common    /fagrd/xfang(nf,2),xcont(3)                            
      common    /mprak/ ista1,iend1,ista2,iend2, &
     &                   ista3,iend3,ista4,iend4 
      common    /myrak/ myrank,nprocs                                  
      common    /scatv/ xegrd(ne,3),flam(ne),fmu(ne),fro(ne),dxol(ne), &
     &                  mvolm(ne)

!                                                                       
!                                                                       
!                                                                       
       complex*16   zwkscv01(3*(ista2-1):3*iend2)                     
       complex*16   zwkscv02(3*(ista2-1):3*iend2)                    
       complex*16   zwkscv03(3*(ista2-1):3*iend2)                    
! 
       complex*16   zwkscv04(3*(ista2-1):3*iend2)                    
       complex*16   zwkscv05(3*(ista2-1):3*iend2)                   
       complex*16   zwkscv06(3*(ista2-1):3*iend2)                  
! 
       complex*16   zwkscv07(3*(ista2-1):3*iend2)                     
       complex*16   zwkscv08(3*(ista2-1):3*iend2)                    
       complex*16   zwkscv09(3*(ista2-1):3*iend2)                   
! 
       complex*16   zwkscv10(3*(ista2-1):3*iend2)                   
       complex*16   zwkscv11(3*(ista2-1):3*iend2)                  
       complex*16   zwkscv12(3*(ista2-1):3*iend2)                 
! 
!                                                                       
       complex*16  zwkscv01t(ne3),zwkscv02t(ne3),zwkscv03t(ne3)  
       complex*16  zwkscv04t(ne3),zwkscv05t(ne3),zwkscv06t(ne3) 
       complex*16  zwkscv07t(ne3),zwkscv08t(ne3),zwkscv09t(ne3)     
       complex*16  zwkscv10t(ne3),zwkscv11t(ne3),zwkscv12t(ne3)     
!                                                                       
       dimension  xf(3),yf(3),zgru(3,3,4)                           
       dimension  zwksc(nf3,ne3,4)  
       dimension  zhfrv(nf3,ne),zhfrs(nf3,ne6),zhfrg(nf3,ne3) 
       dimension  zopea(nf3,nf3)
! 
!                                                                   
       dimension   idisp4(0:40),ircnt4(0:40)                            
!                                                                       
       idisp4(myrank)=3*(ista2-1)                                       
       ircnt4(myrank)=3*(iend2-ista2+1)                                 
!                                                                       
         do ii=0,nprocs-1                                               
         call mpi_bcast(idisp4(ii),1,mpi_integer,ii, &               
     &                  MPI_COMM_WORLD,ierr)                            
         call mpi_bcast(ircnt4(ii),1,mpi_integer,ii, &              
     &                  MPI_COMM_WORLD,ierr)                            
         enddo                                                          
!                                                                       
!                                                                       
         iist4=idisp4(myrank)+1                                         
         iict4=ircnt4(myrank)                                           
!                                                                       
!                                                                       
!                                                                       
!                                                                       
       nnf3=nnf*3                                                       
       nne3=nne*3
       nne6=nne*6                                                       
!                                                                       
!                                                                       
       if(myrank.eq.0) then
        open(unit=442,file='proc')
        open(unit=444,file='scateg')
        open(unit=446,file='zwksc')
       endif

!                                                                       
!         if(myrank.eq.0) then
!           do   ii=1, nne
!           write(444,*)xegrd(ii,1),xegrd(ii,2),xegrd(ii,3)
!           enddo
!         endif
!
!
!
!                                                                       
!                                                                       
          do   ii=1, nnf 
!              do   ii=1, 1   
!                                                                       
!        if(myrank.eq.0)   write(*,*)' ii=',ii, 'nnf=',nnf              
!                                                                       
         xf(1)=xfgrd(ii,1)                                              
         xf(2)=xfgrd(ii,2)                                              
         xf(3)=xfgrd(ii,3)                                              
!                                                                       
         do   jj=ista2, iend2                                           
          yf(1)=xegrd(jj,1)         
          yf(2)=xegrd(jj,2)        
          yf(3)=xegrd(jj,3)       
!                                                                       
          if(infc1.eq.0) then                                           
               call  green(zgru,xf,yf)                                  
             else                                                       
              call  greensd(zgru,xf,yf )                                
          endif                                                         
!                                                                       
!                                                                       
!           if(myrank.eq.1) then
!           write(442,*)'myrank=',myrank, 'jj=',jj
!           write(442,*)' xf=',xf(1),xf(2),xf(3)
!           write(442,*)' yf=',yf(1),yf(2),yf(3)
!             do  iip=1, 3
!              do  jjp=1, 3
!              write(442,*)iip,jjp,cdabs(zgru(iip,jjp,1))
!              enddo
!             enddo
!           endif
!
!                         
!                                              
!                                                                       
           do  ll=1,3                                                   
            zwkscv01(3*(jj-1)+ll)=zgru(1,ll,1)                   
            zwkscv02(3*(jj-1)+ll)=zgru(2,ll,1)                  
            zwkscv03(3*(jj-1)+ll)=zgru(3,ll,1)
!
            zwkscv04(3*(jj-1)+ll)=zgru(1,ll,2)                      
            zwkscv05(3*(jj-1)+ll)=zgru(2,ll,2)                     
            zwkscv06(3*(jj-1)+ll)=zgru(3,ll,2)
!
            zwkscv07(3*(jj-1)+ll)=zgru(1,ll,3)                    
            zwkscv08(3*(jj-1)+ll)=zgru(2,ll,3)                   
            zwkscv09(3*(jj-1)+ll)=zgru(3,ll,3)
! 
            zwkscv10(3*(jj-1)+ll)=zgru(1,ll,4)                    
            zwkscv11(3*(jj-1)+ll)=zgru(2,ll,4)                   
            zwkscv12(3*(jj-1)+ll)=zgru(3,ll,4)
! 
!                             
           enddo                                                        
          enddo                                                         
!                                                                       
!                                                                       
!                                                                       
        call mpi_allgatherv(zwkscv01(iist4),iict4,mpi_complex16,&      
     &   zwkscv01t,ircnt4,idisp4,mpi_complex16,MPI_COMM_WORLD,ierr)    
        call mpi_allgatherv(zwkscv02(iist4),iict4,mpi_complex16,&      
     &   zwkscv02t,ircnt4,idisp4,mpi_complex16,MPI_COMM_WORLD,ierr)    
        call mpi_allgatherv(zwkscv03(iist4),iict4,mpi_complex16,&     
     &   zwkscv03t,ircnt4,idisp4,mpi_complex16,MPI_COMM_WORLD,ierr)    
!
        call mpi_allgatherv(zwkscv04(iist4),iict4,mpi_complex16,&      
     &   zwkscv04t,ircnt4,idisp4,mpi_complex16,MPI_COMM_WORLD,ierr)    
        call mpi_allgatherv(zwkscv05(iist4),iict4,mpi_complex16,&     
     &   zwkscv05t,ircnt4,idisp4,mpi_complex16,MPI_COMM_WORLD,ierr)    
        call mpi_allgatherv(zwkscv06(iist4),iict4,mpi_complex16,&      
     &   zwkscv06t,ircnt4,idisp4,mpi_complex16,MPI_COMM_WORLD,ierr)   
!

        call mpi_allgatherv(zwkscv07(iist4),iict4,mpi_complex16,&     
     &   zwkscv07t,ircnt4,idisp4,mpi_complex16,MPI_COMM_WORLD,ierr)   
        call mpi_allgatherv(zwkscv08(iist4),iict4,mpi_complex16,&     
     &   zwkscv08t,ircnt4,idisp4,mpi_complex16,MPI_COMM_WORLD,ierr)   
        call mpi_allgatherv(zwkscv09(iist4),iict4,mpi_complex16,&     
     &   zwkscv09t,ircnt4,idisp4,mpi_complex16,MPI_COMM_WORLD,ierr)   
!
        call mpi_allgatherv(zwkscv10(iist4),iict4,mpi_complex16,&     
     &   zwkscv10t,ircnt4,idisp4,mpi_complex16,MPI_COMM_WORLD,ierr)   
        call mpi_allgatherv(zwkscv11(iist4),iict4,mpi_complex16,&     
     &   zwkscv11t,ircnt4,idisp4,mpi_complex16,MPI_COMM_WORLD,ierr)   
        call mpi_allgatherv(zwkscv12(iist4),iict4,mpi_complex16,&     
     &   zwkscv12t,ircnt4,idisp4,mpi_complex16,MPI_COMM_WORLD,ierr)  
! 
                                                                       
        call mpi_barrier(MPI_COMM_WORLD,ierr)                           
!                                                                       
          do  jj=1, nne3
!                                                
           zwksc(3*ii-2,jj, 1)=zwkscv01t(jj)  
           zwksc(3*ii-1,jj, 1)=zwkscv02t(jj) 
           zwksc(3*ii  ,jj, 1)=zwkscv03t(jj)  
! 
           zwksc(3*ii-2,jj, 2)=zwkscv04t(jj) 
           zwksc(3*ii-1,jj, 2)=zwkscv05t(jj)
           zwksc(3*ii  ,jj, 2)=zwkscv06t(jj)
! 
           zwksc(3*ii-2,jj, 3)=zwkscv07t(jj)   
           zwksc(3*ii-1,jj, 3)=zwkscv08t(jj)  
           zwksc(3*ii  ,jj, 3)=zwkscv09t(jj) 
! 
           zwksc(3*ii-2,jj, 4)=zwkscv10t(jj)  
           zwksc(3*ii-1,jj, 4)=zwkscv11t(jj) 
           zwksc(3*ii  ,jj, 4)=zwkscv12t(jj) 
! 
          enddo                                                         
!                                                                       
!                                                                       
       enddo                                                            
!                                                                        
!           if(myrank.eq.0) then
!             do   iiq=1, nnf3  
!             do   jjq=1, nne3
!             write(446,*)iiq,jjq,cdabs(zwksc(iiq,jjq,1))
!             enddo
!            write(446,*)'    '
!             enddo
!             close(unit=446)
!           endif
!                                                                        
!   
      do    jj=1, nne
! 
         eflm=dsqrt ( flam(jj) )
         efmu=dsqrt ( fmu(jj)  )
         erho=dsqrt ( fro(jj)  )*omega
         efmu2=dsqrt(2.0d0)*efmu
!
         do    ii=1, nnf
          do     kk=1, 3
! 
          kki=kk-3
! 
         zhfrv(3*ii+kki ,jj)=( zwksc(3*ii+kki,  3*jj-2, 2)+  &       
      &  zwksc(3*ii+kki, 3*jj-1, 3)+ zwksc(3*ii+kki, 3*jj, 4) )*eflm 
! 
         zhfrs(3*ii+kki,6*(jj-1)+1) =zwksc(3*ii+kki, 3*jj-2 ,2)*efmu2
         zhfrs(3*ii+kki,6*(jj-1)+2) = (zwksc(3*ii+kki,3*jj-1,2)+&
         &        zwksc(3*ii+kki, 3*jj-2 ,3) )*efmu
         zhfrs(3*ii+kki,6*(jj-1)+3) =(zwksc(3*ii+kki,3*jj,2)+ &
          &       zwksc(3*ii+kki, 3*jj-2 ,4) )*efmu
         zhfrs(3*ii+kki, 6*(jj-1)+4) = zwksc(3*ii+kki, 3*jj-1 ,3)*efmu2
         zhfrs(3*ii+kki, 6*(jj-1)+5) = (zwksc(3*ii+kki,3*jj,3)+ &
          &       zwksc(3*ii+kki, 3*jj-1 ,4) )*efmu
         zhfrs(3*ii+kki, 6*(jj-1)+6) = zwksc(3*ii+kki, 3*jj  ,4)*efmu2
! 
!  
!  
        zhfrg(3*ii+kki, 3*jj-2    ) = zwksc(3*ii+kki, 3*jj-2 ,1)*erho
        zhfrg(3*ii+kki, 3*jj-1    ) = zwksc(3*ii+kki, 3*jj-1 ,1)*erho
        zhfrg(3*ii+kki, 3*jj      ) = zwksc(3*ii+kki, 3*jj   ,1)*erho
! 
            enddo
! 
         enddo
        enddo
! 
!
! 
! 
           do  ii=1, nnf3
            do  jj=1, nnf3
! 
            zwk1=(0.0d0,0.0d0)
            zwk2=(0.0d0,0.0d0)
            zwk3=(0.0d0,0.0d0)
! 
            zopea(ii,jj)=(0.0d0,0.0d0)
! 
             do  kk=ista2, iend2
             zwk1=zwk1+zhfrv(ii,kk)*zhfrv(jj,kk)
             enddo
! 
             do  kk=ista3, iend3
             zwk2=zwk2+zhfrg(ii,kk)*zhfrg(jj,kk)
             enddo
! 
             do  kk=ista4,iend4
             zwk3=zwk3+zhfrs(ii,kk)*zhfrs(jj,kk)
             enddo
! 
             call  mpi_barrier(MPI_COMM_WORLD,ierr)                  
             call  mpi_allreduce(zwk1,zwkat1,1,mpi_complex16,  &
                            &  MPI_SUM, MPI_COMM_WORLD,ierr)
             call  mpi_allreduce(zwk2,zwkat2,1,mpi_complex16,  &
                            &  MPI_SUM, MPI_COMM_WORLD,ierr)
             call  mpi_allreduce(zwk3,zwkat3,1,mpi_complex16,  &
                            &  MPI_SUM, MPI_COMM_WORLD,ierr)
! 
             call  mpi_barrier(MPI_COMM_WORLD,ierr)                  
            zopea(ii,jj)=-zwkat1+zwkat2-zwkat3
! 
!            zopea(ii,jj)=zwkat2
          enddo
         enddo
!                                                                       
!                                                                       
!           if(myrank.eq.0) then
!            open(unit=441,file='scato')
!            do ii=1, nnf3
!               do   jj=1, nnf3
!             write(441,*)ii,jj,cdabs(zopea(ii,jj))
!             enddo
!             write(441,*)'   '
!            enddo
!            close(unit=441)
!            endif
! 
! 
! 
!                                                                       
!                                                                       
!                                                                       
        return                                                          
        end                                                             
!                                                                       
!                                                                       
!                                                                       
!                                                                       
! * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
      subroutine  proj(zalproj,zbbproj,zbtproj)                         
! * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
!                                                                       
!       use struct_type
! 
       implicit  real*8(a-h,o-y)                                        
       implicit  integer(i-n)  
       implicit  complex*16(z)                                          
!                                                                       
       parameter  (np=500)                                              
       parameter  (nf=np)                                               
       parameter  (nf3=np*3)                                            
       parameter  (ne=2000)                                             
       parameter  (ne3=ne*3)                                            
       parameter  (nmax=nf3)                                            
!                                                                       
!                                                                       
       common    /media/ro,ct,cl,rmu,ray,rtl,rlam                       
       common    /dynam/omega,dka,dkb,eps                               
       common    /fagrd/xfang(nf,2),xcont(3)                            
       common    /pjvct/zp(nf3),zps(nf3),zpst(nf3),zdapr(nf)            
       common    /sjvct/zs(nf3),zss(nf3),zsst(nf3),zt(nf3),&            
     &                  zdbpr(nf),zdtpr(nf)                             
       common    /fsgrd/xfgrd(nf,3),nnf,nne 
!                                                                       
!                                                                       
       dimension  zalproj(nf,3,3),zbbproj(nf,3,3),zbtproj(nf,3,3)       
       dimension  xf(3),yf(3),zgru1(3,3,4),zgru2(3,3,4),zwkbv(3,3)  
!                                                                       
!                                                                       
!                                                                       
       open(unit=23,file='plam-proj', status='unknown')                 
!       write(23,*)'  check of subroutine proj   '                       
!       write(23,*)'xcont=',(xcont(kk),kk=1,3)  
!            
       open(unit=24,file='chk-projp')
!                                                                       
!                                                                       
!                                                                       
      zi=(0.0d0,1.0d0)                                                  
      pi=3.14159265358979323846d0                                       
!                                                                       
!                                                                       
        do   ii=1, nnf                                                  
!                                                                       
!            write(23,*)' ii=',ii, 'nnf=',nnf                              
!
!                                                                       
!                                                                       
         dx1=xfgrd(ii,1)-xcont(1)                                       
         dx2=xfgrd(ii,2)-xcont(2)
         dx3=xfgrd(ii,3)-xcont(3)                                       
! 
!                                                                       
        rra=dsqrt( dx1**2 +dx2**2 +dx3**2)                              
!                                                                       
        theta=dacos (-dx3/rra )                                        
        phi =datan2 (dx2, dx1 )                                         
!                                                                       
!            write(23,*)' ii=',ii  
!            write(23,*)'xcont=',(xcont(kk),kk=1,3)              
!            write(23,*)'fgrd=',(xfgrd(ii,kk),kk=1,3)           
!            write(23,*)' dx=',dx1,dx2,dx3   
!            write(23,*)' rra=',rra                         
!            write(23,*)' theta=',theta,' phi=',phi            
!                                                                       
!                                                                       
!                                                                       
        zgal =4.0d0*pi*rra*cdexp( -zi*rra*dka)                          
        zgbt =4.0d0*pi*rra*cdexp( -zi*rra*dkb)                          
!                                                                       
!           write(23,*)' dka=',dka, ' dkb=',dkb, ' rra=',rra               
!           write(23,*)'    '                                              
!                                                                       
!                                                                       
!                                                                       
!     ** asymptotic P wave **                                           
      sal=dka*dsin (theta )                                             
      sal2=sal*sal                                                      
      zgam=-zi*dka*dcos ( theta )                                       
      zrnu=-zi*dsqrt( dkb*dkb-sal2)                                     
      zrnu2=zrnu*zrnu                                                   
      zfk=(2.0d0*sal2-dkb*dkb)**2-4.0d0*sal2*zgam*zrnu                  
      zfkc=1.0d0/(rmu*zfk)                                          
!                                                                       
      zampl11=zgam*zgam*(sal2+zrnu2)*zfkc                          
      zampl21=sal* 2.0d0*zgam*zgam*zrnu*zfkc                      
      zampl12=-sal*zgam*(sal2+zrnu2)*zfkc                        
      zampl22=-2.0d0*zrnu*zgam*sal2*zfkc                        
!                                         
!                                                                       
      if(dabs(theta).le.1.0d-3)then                                     
        zps3=(1.0d0,0.0d0)                                     
        zpst3=(0.0d0,0.0d0)                                   
        zdapr(ii)=2.0d0*zampl11                              
      else
        zka=zampl12/zampl22 
        zps3=-zi*zka*dsin(theta)                             
        zpst3=1.0d0/(zi*zka)                                  
        zdapr(ii)=2.0d0*zampl22/( dsin (theta)**2)                    
      endif                                                             
!                                                                       
      zp(3*(ii-1)+1)=dcos(phi)*dsin(theta)                              
      zp(3*(ii-1)+2)=dsin(phi)*dsin(theta)                              
      zp(3*(ii-1)+3)=-dcos(theta)                                    
!                                                                       
      zps(3*(ii-1)+1)=zp( 3*(ii-1)+1)                                   
      zps(3*(ii-1)+2)=zp (3*(ii-1)+2)                                   
      zps(3*(ii-1)+3)=zps3                                             
!                                                                       
      zpst(3*(ii-1)+1)=dcos(phi )                                       
      zpst(3*(ii-1)+2)=dsin(phi)                                        
      zpst(3*(ii-1)+3)=zpst3                                           
!                                                                       
!       write(23,*)' zps3=',cdabs(zps3), ' zpst3=',cdabs(zpst3)
!                                                                       
!    ** asymptotic S wave **                                            
      sat=dkb*dsin(theta)                                               
      sat2=sat*sat                                                      
      if(sat.ge.dka) then                                               
      zgam=dsqrt( sat2- dka*dka)                                        
      else                                                              
      zgam=-zi*dsqrt( dka*dka -sat2)                                    
      endif                                                             
!                                                                       
      zrnu=-zi*dkb*dcos ( theta )                                       
      zrnu2=zrnu*zrnu                                                   
!                                                                       
      zfk=(2.0d0*sat2-dkb*dkb)**2-4.0d0*sat2*zgam*zrnu       
      zfkc=1.0d0/(rmu*zfk)                    
!                                                                       
!                                                                       
      zampt11=-2.0d0*zgam*zrnu*sat2*zfkc                 
      zampt21=-sat*zrnu*(sat2+zrnu2)*zfkc                 
      zampt12=2.0d0*sat*zgam*zrnu2*zfkc                    
      zampt22=zrnu2*(sat2+zrnu2)*zfkc                       
      zampt33=1.0d0/rmu                                                 
!                                                                       
!     
                                                     
      rzampt22=cdabs(zampt22)                                          
      if(rzampt22.gt.1.0d-10)then
         zkb=zampt12/zampt22
         zss3=-zi*zkb*dcos(theta) 
         ffc=1.0d0
         if(cdabs(zkb).gt.1.0d-10 )then                            
         zsst3=1.0d0/(zi*zkb)
         dcpt=dcos(phi)
         dspt=dsin(phi)
         else
         zsst3=(1.0d0,0.0d0)
         dcpt=0.0d0
         dspt=0.0d0
         endif
!
         zdbpr(ii)=2.0d0*zampt22/( dcos(theta)**2 )     
!
         endif
!
!
      if(rzampt22.lt.1.0d-10)then
!
         zss3=1.0d0/dsin(theta)
         dcpt=1.0d0 
         dspt=1.0d0 
         zsst3=(0.0d0,0.0d0)
         zdbpr(ii)=2.0d0*zampt11 
         ffc=0.0d0
!
      endif
!          write(23,*)' dcpt dspt zsst =',dcpt,dspt,zsst3
!    
         zs(3*(ii-1)+1)=dcos(phi)*dcos(theta)                    
         zs(3*(ii-1)+2)=dsin(phi)*dcos(theta)                   
         zs(3*(ii-1)+3)=dsin(theta)                            
!                                                             
         zss(3*(ii-1)+1)=zs( 3*(ii-1)+1)*ffc                 
         zss(3*(ii-1)+2)=zs( 3*(ii-1)+2)*ffc                
         zss(3*(ii-1)+3)=zss3                               
!                                                                       
         zsst(3*(ii-1)+1)=dcpt                                      
         zsst(3*(ii-1)+2)=dspt                                      
         zsst(3*(ii-1)+3)=zsst3                                           
!                                                                       
!
!
!                                                                       
!                                                                 
!       write(23,*)' dcpt dspt zsst =',dcpt,dspt,zsst3
!       write(23,*)' zss3=',cdabs(zss3), ' zsst3=',cdabs(zsst3)
!       write(23,*)' ffc=',ffc
       zt(3*(ii-1)+1)=dsin (phi)                                      
       zt(3*(ii-1)+2)=-dcos (phi)                                    
       zt(3*(ii-1)+3)=(0.0d0,0.0d0)                                 
!                                                                       
!                                                                       
        zalprojm=zps(3*ii-2)*zsst(3*ii-2)+zps(3*ii-1)*zsst(3*ii-1) & 
     &          + zps(3*ii)*zsst(3*ii)                                  
        zbbprojm=zss(3*ii-2)*zpst(3*ii-2)+zss(3*ii-1)*zpst(3*ii-1) & 
     &          + zss(3*ii)*zpst(3*ii)                                  
                                                                        
!                                                                       
!        write(23,*)' zps1=',zps(3*ii-2)
!        write(23,*)' zps2=',zps(3*ii-1)
!        write(23,*)' zps3=',zps(3*ii  )
!
!        write(23,*)' zsst1=',zsst(3*ii-2)
!        write(23,*)' zsst2=',zsst(3*ii-1)
!        write(23,*)' zsst3=',zsst(3*ii  )
!                                                                       
!          write(23,*)' zalprojm=',cdabs(zalprojm)                         
!          write(23,*)' zbbprojm=',cdabs(zbbprojm)                         
!          write(23,*)'zgal=',zgal
!         write(23,*)'zgbt=',zgbt
!                                                                       
!                                                                       
!                                                                       
        
         do   iit=1, 3                                                  
          do   jjt=1, 3                                                 
          ilo=3*(ii-1)                                                  
          zalproj(ii,iit,jjt)=zgal*zps(ilo+iit)*zsst(ilo+jjt)/zalprojm  
          zbbproj(ii,iit,jjt)=zgbt*zss(ilo+iit)*zpst(ilo+jjt)/zbbprojm  
          zbtproj(ii,iit,jjt)=zgbt*zt(ilo+iit)*zt(ilo+jjt)              
          enddo                                                         
         enddo                                                          
!   
!         do   iit=1, 3
!          do jjt=1, 3
!          write(23,*) iit,jjt,cdabs(zalproj(ii,iit,jjt))
!          enddo
!         enddo
!
!
!                                                                       
        enddo                                                           
!                                                                       
        close(unit=23)                                                  
!                                                                       
! 
!     **** test for proj  ***
! 
         idbgproj=0
         if(idbgproj.eq.1) then
! 
!           nnfselct=nnf
          nnfselct=1
          do  iifp=50, 50

!                                                                       
         dx1=xfgrd(iifp,1)-xcont(1)                                  
         dx2=xfgrd(iifp,2)-xcont(2)
         dx3=xfgrd(iifp,3)-xcont(3)                                  
!
! 
         rra=dsqrt ( dx1*dx1+ dx2*dx2 + dx3*dx3 )
         theta=dacos (-dx3/rra )                                     
         phi  =datan2( dx2, dx1 )                                   
!                                      
         sal=dka*dsin (theta)  
! 
! 
          write(24,*)' grid point iifp=',iifp
!        
         do   kk=1, 3
          xf(kk)=xfgrd(iifp,kk)
         enddo
! 
         write(24,*)' xf  =',xf(1),xf(2),xf(3)
         write(24,*)'xcont=',xcont(1),xcont(2),xcont(3)
         write(24,*)'dx=',dx1,dx2,dx3
         write(24,*)' rra=',rra
         write(24,*)'theta=',theta,' phi=',phi
! 
         do   ii=1, 3
          do    jj=1, 3
          zgru2(ii,jj,1)=zps(3*(iifp-1)+ii)*zp(3*(iifp-1)+jj)*   &
     &                  zdapr(iifp)
          zgru2(ii,jj,2)=-zi*dka* zps(3*(iifp-1)+ii)*zp(3*(iifp-1)+jj)  &
     &                   *zp(3*(iifp-1)+1)*zdapr(iifp)
          zgru2(ii,jj,3)=-zi*dka* zps(3*(iifp-1)+ii)*zp(3*(iifp-1)+jj)  &
     &                   *zp(3*(iifp-1)+2)*zdapr(iifp)
          zgru2(ii,jj,4)=-zi*dka* zps(3*(iifp-1)+ii)*zp(3*(iifp-1)+jj)  &
     &                   *zp(3*(iifp-1)+3)*zdapr(iifp)
          enddo
         enddo
!         
          call   greensda(zgru1,xf,xcont,11)
! 
! 
!          
           write(24,*)' P-wave component (mono-pole)'
! 
           do  ii=1, 3
            do  jj=1, 3
            write(24,*) ii,jj
            write(24,*)' greensda=',zgru1(ii,jj,1)
            write(24,*)' decomp  =',zgru2(ii,jj,1)
            enddo
           write(24,*)'    '
           enddo
! 
! 
          write(24,*)'  ++ '
          write(24,*)' P-wave component (dipole 1)'
! 
           do  ii=1, 3
            do  jj=1, 3
            write(24,*) ii,jj
            write(24,*)' greensda=',zgru1(ii,jj,2)
            write(24,*)' decomp  =',zgru2(ii,jj,2)
            enddo
           write(24,*)'    '
           enddo
! 
!  
          write(24,*)'  ++ '
          write(24,*)' P-wave component (dipole 2)'
! 
           do  ii=1, 3
            do  jj=1, 3
            write(24,*) ii,jj
            write(24,*)' greensda=',zgru1(ii,jj,3)
            write(24,*)' decomp  =',zgru2(ii,jj,3)
            enddo
           write(24,*)'    '
           enddo
! 
         write(24,*)'  ++ '
          write(24,*)' P-wave component (dipole 3)'
! 
           do  ii=1, 3
            do  jj=1, 3
            write(24,*) ii,jj
            write(24,*)' greensda=',zgru1(ii,jj,4)
            write(24,*)' decomp  =',zgru2(ii,jj,4)
            enddo
           write(24,*)'    '
           enddo
! 
! 
! 
!      sat=dkb*dsin (theta)  
!
!      sat=dkb*dsin(theta)                                               
!      sat2=sat*sat                                                      
!      if(sat.ge.dka) then                                               
!      zgam=dsqrt( sat2- dka*dka)                                        
!      else                                                              
!      zgam=-zi*dsqrt( dka*dka -sat2)                                    
!      endif                                                             
!                                                                       
!      zrnu=-zi*dkb*dcos ( theta )                                       
!      zrnu2=zrnu*zrnu                                                   
!                                                                       
!      zfk=(2.0d0*sat2-dkb*dkb)**2-4.0d0*sat2*zgam*zrnu                  
!      zfkc=1.0d0/(rmu*zfk)                                               
!                                                                       
!                                                                       
         do   ii=1, 3
          do    jj=1, 3
           zgru2(ii,jj,1)=zss(3*(iifp-1)+ii)*zs(3*(iifp-1)+jj) &
              &                 *zdbpr(iifp)
!
           zgru2(ii,jj,2)=-zi*dkb*zss(3*(iifp-1)+ii)*zs(3*(iifp-1)+jj) &
              &               *zp(3*(iifp-1)+1)*zdbpr(iifp)
           zgru2(ii,jj,3)=-zi*dkb*zss(3*(iifp-1)+ii)*zs(3*(iifp-1)+jj) &
              &               *zp(3*(iifp-1)+2)*zdbpr(iifp)
           zgru2(ii,jj,4)=-zi*dkb*zss(3*(iifp-1)+ii)*zs(3*(iifp-1)+jj) &
              &               *zp(3*(iifp-1)+3)*zdbpr(iifp)
!
!
          enddo
         enddo
!         
          call   greensda(zgru1,xf,xcont,22)
! 
! 
! 
           write(24,*)' SV-wave component (mono-pole)' 
          do  ii=1, 3
           do  jj=1, 3
           write(24,*)' ii jj=',ii, jj
            write(24,*)' greensda=',zgru1(ii,jj,1)
            write(24,*)' decomp  =',zgru2(ii,jj,1)
           enddo
          write(24,*)'    '
          enddo
! 
!
          write(24,*)'  ++ '
!      sat=dkb*dsin(theta)                                               
!      sat2=sat*sat                                                      
!      if(sat.ge.dka) then                                               
!      zgam=dsqrt( sat2- dka*dka)                                        
!      else                                                              
!      zgam=-zi*dsqrt( dka*dka -sat2)                                    
!      endif                                                             
!                                                                       
!      zrnu=-zi*dkb*dcos ( theta )                                       
!      zrnu2=zrnu*zrnu                                                   
!                                                                       
!      zfk=(2.0d0*sat2-dkb*dkb)**2-4.0d0*sat2*zgam*zrnu                  
!      zfkc=1.0d0/(rmu*zfk)                                               
!                                                                       
!                                                                       
          write(24,*)' SV-wave component (dipole 1)'
! 
          do  ii=1, 3
            do  jj=1, 3
            write(24,*)' ii jj=',ii, jj
            write(24,*)' greensda=',zgru1(ii,jj,2)
            write(24,*)' decomp  =',zgru2(ii,jj,2)
            enddo
           write(24,*)'    '
           enddo
!
! 
          write(24,*)'  ++ '
          write(24,*)' SV-wave component (dipole 2)'
! 
          do  ii=1, 3
            do  jj=1, 3
            write(24,*)' ii jj=',ii, jj
            write(24,*)' greensda=',zgru1(ii,jj,3)
            write(24,*)' decomp  =',zgru2(ii,jj,3)
            enddo
           write(24,*)'    '
           enddo
! 
         write(24,*)'  ++ '
          write(24,*)' SV-wave component (dipole 3)'
! 
          do  ii=1, 3
            do  jj=1, 3
            write(24,*)' ii jj=',ii, jj
            write(24,*)' greensda=',zgru1(ii,jj,4)
            write(24,*)' decomp  =',zgru2(ii,jj,4)
            enddo
           write(24,*)'    '
           enddo
! 
! 
        do   ii=1, 3
          do    jj=1, 3
          zgru2(ii,jj,1)=zt(3*(iifp-1)+ii)*zt(3*(iifp-1)+jj)*zdtpr(iifp)
          zgru2(ii,jj,2)=-zi*dkb*zt(3*(iifp-1)+ii)*zt(3*(iifp-1)+jj) &
         &               *zp(3*(iifp-1)+1)*zdtpr(iifp)
          zgru2(ii,jj,3)=-zi*dkb*zt(3*(iifp-1)+ii)*zt(3*(iifp-1)+jj) &
         &               *zp(3*(iifp-1)+2)*zdtpr(iifp)
          zgru2(ii,jj,4)=-zi*dkb*zt(3*(iifp-1)+ii)*zt(3*(iifp-1)+jj) &
         &               *zp(3*(iifp-1)+3)*zdtpr(iifp)
         enddo
        enddo
!
!         write(151,*)'  check '          
         call   greensda(zgru1,xf,xcont,33)
! 
! 
! 
            write(24,*)' SH-wave component (mono-pole) ' 
           do  ii=1, 3
            do  jj=1, 3
            write(24,*)' ii jj=',ii, jj
           write(24,*)' greensda=',zgru1(ii,jj,1)
           write(24,*)' decomp  =',zgru2(ii,jj,1)
           enddo
          write(24,*)'    '
          enddo
! 
! 
          write(24,*)'  ++ '
          write(24,*)' SH-wave component (dipole 1)'
! 
          do  ii=1, 3
            do  jj=1, 3
            write(24,*)' ii jj=',ii, jj
            write(24,*)' greensda=',zgru1(ii,jj,2)
            write(24,*)' decomp  =',zgru2(ii,jj,2)
            enddo
           write(24,*)'    '
           enddo
! 
          write(24,*)'  ++ '
          write(24,*)' SH-wave component (dipole 2)'
! 
          do  ii=1, 3
            do  jj=1, 3
            write(24,*)' ii jj=',ii, jj
            write(24,*)' greensda=',zgru1(ii,jj,3)
            write(24,*)' decomp  =',zgru2(ii,jj,3)
            enddo
           write(24,*)'    '
           enddo
! 
! 
          write(24,*)'  ++ '
          write(24,*)' SH-wave component (dipole 3)'
! 
          do  ii=1, 3
            do  jj=1, 3
            write(24,*)' ii jj=',ii, jj
            write(24,*)' greensda=',zgru1(ii,jj,4)
            write(24,*)' decomp  =',zgru2(ii,jj,4)
            enddo
           write(24,*)'    '
           enddo
! 
!
           write(24,*)'      '
           write(24,*)'  orthogonality of the direction vector'
! 
        zalprod=zps(3*iifp-2)*zpst(3*iifp-2)  & 
     &          +zps(3*iifp-1)*zpst(3*iifp-1) &
     &          +zps(3*iifp  )*zpst(3*iifp  )
        zbbprod=zss(3*iifp-2)*zsst(3*iifp-2)  & 
     &         +zss(3*iifp-1)*zsst(3*iifp-1)  &
     &          +zss(3*iifp  )*zsst(3*iifp  ) 
! 
          write(24,*)' zalprod=',cdabs( zalprod )
          write(24,*)' zbbprod=',cdabs( zbbprod )
! 
          write(24,*)'      '
          write(24,*)' +  +     '
! 
!       
         enddo
! 
! 
! 
! 
           close(unit=24)
! 
        endif
!                                                                       
!                                           
!                                                                       
         return                                                         
 8000    stop                                                           
         end                                                            
!                                                                       
!                                                                       
!                                                                       
!                                                                       
!                                                                       
!                                                                       
!                                                                       
! * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
      subroutine  opera(zopea, zopeaf,zwkvl, zwkvr, zwork,rwork,&
     &            zalproj,zbbproj,zbtproj,zwkprojl, zwkprojr,iawave)  
! * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
! 
!
! 
!                                                                       
       implicit  real*8(a-h,o-y)
       implicit  integer (i-n)                                        
       implicit  complex*16(z)                                          
!                                                                       
       parameter  (np=500)                                              
       parameter  (nf=np)                                               
       parameter  (nf3=np*3)
       parameter  (nf12=nf*12)                                       
       parameter  (ne=2000)                                             
       parameter  (ne3=ne*3)                                            
       parameter  (nmax=nf3)                                            
       parameter  (lwork=5*nmax)                                        
       parameter  (liwork=2*nmax  )                                     
       parameter  (nbf=10)
!                                                                       
!                                                                       
       common    /media/ro,ct,cl,rmu,ray,rtl,rlam                       
       common    /dynam/omega,dka,dkb,eps                               
       common    /fsgrd/xfgrd(nf,3), nnf,nne         
       common    /cutof/epst                                            
       common    /inpar/divk,deltk,rmax,fdivk,fdeltk,infc1              
       common    /fagrd/xfang(nf,2),xcont(3)                            
!                                                                       
!                                                                       
       dimension  zopea(nf3,nf3),zopeaf(nf3,nf3)              
!                                                                       
       dimension  zrzw(nf3), zwkvl(nf3,nf3),zwkvr(nf3,nf3)              
       dimension  zwork(lwork),rwork(liwork)                            
!                                                                       
       dimension  zalproj(nf,3,3),zbbproj(nf,3,3),zbtproj(nf,3,3)       
       dimension  zwkprojl(nf,3,3),zwkprojr(nf,3,3)                     
!                                                                       
       dimension  zaawk(3,3),zlawk(3,3),zrawk(3,3)
       dimension  zwrwk(3,3),zbbwk(3,3)  
!                                                                      
!                                                                       
!                                                                       
!                                                                       
       nnf3=nnf*3                                                       
       nne3=nne*3                                                       
!                                                                       
!       write(*,*)'         '                                           
!       write(*,*)'         '                                           
!       write(*,*)' ** check opera nnf3=',nnf3, ' nne3=',nne3           
!                                                                       
       il=iawave/10                                                     
       ir=mod(iawave,10)                                                
!                                                                       
       if(il.eq.1)then 
!        ccp=0.0d0                                                 
        do  ii=1, nnf                                                  
           do iij=1, 3                                                
           do  iik=1, 3                                              
             zwkprojl(ii,iij,iik)=zalproj(ii,iij,iik)  
!             ccp=ccp+cdabs( zalproj(ii,iij,iik) )                    
           enddo                                                        
           enddo                                                       
         enddo   
!            write(*,*)' opera ccp=',ccp                                                        
       endif                                                            
!                                                                       
       if(il.eq.2)then                                                  
        do  ii=1, nnf                                                    
         do iij=1, 3                                                    
         do  iik=1, 3                                                   
         zwkprojl(ii,iij,iik)=zbbproj(ii,iij,iik)                       
         enddo                                                          
         enddo                                                          
       enddo                                                            
      endif                                                            
!                                                                       
       if(il.eq.3)then                                                  
        do  ii=1, nnf                                                 
         do iij=1, 3                                                    
         do  iik=1, 3                                                   
         zwkprojl(ii,iij,iik)=zbtproj(ii,iij,iik)                       
         enddo                                                          
         enddo                                                          
       enddo                                                            
       endif                                                            
!                                                                        
! 
! 
! 
! 
! 
 
       if(ir.eq.1)then                                                  
        do  ii=1, nnf                                                
           do iij=1, 3                                               
           do  iik=1, 3                                              
             zwkprojr(ii,iij,iik)=zalproj(ii,iij,iik)               
           enddo                                                   
           enddo                                                  
         enddo                                                   
       endif                                                            
!                                                                       
       if(ir.eq.2)then                                                  
        do  ii=1, nnf                                           
         do iij=1, 3                                                    
         do  iik=1, 3                                                   
         zwkprojr(ii,iij,iik)=zbbproj(ii,iij,iik)                       
         enddo                                                          
         enddo                                                          
       enddo                                                            
      endif                                                            
!                                                                       
       if(ir.eq.3)then                                                  
        do  ii=1, nnf                                          
         do iij=1, 3                                                    
         do  iik=1, 3                                                   
         zwkprojr(ii,iij,iik)=zbtproj(ii,iij,iik)                       
         enddo                                                          
         enddo                                                          
       enddo                                                            
       endif                                                            
!                                                                        
! 
! 
        if(myrank.eq.0)then
         open(unit=301,file='plam-projl')
         open(unit=302,file='plam-projr')
         do  iia=1, nnf
         write(301,*) '*iia=',iia, ' *projl*'
         write(302,*) '*iia=',iia, ' *projr*'
         do   jja=1, 3
         do   kka=1, 3
         write(301,*)jja,kka,cdabs(zwkprojl(iia,jja,kka))
         write(302,*)jja,kka,cdabs(zwkprojl(iia,jja,kka))
         enddo
         enddo
         write(301,*)' *** '
         write(302,*)' *** '
         enddo
         close(unit=301)
         close(unit=302)
        endif
! 
! 
!                                                                       
         do   ii=1, nnf                                                 
          do   jj=1, nnf
!                                                
            do    ksi=1, 3                                              
             do    ksj=1, 3
              zaawk(ksi,ksj)=zopea(3*(ii-1)+ksi,3*(jj-1)+ksj)
              zlawk(ksi,ksj)=zwkprojl(ii,ksi,ksj)
              zrawk(ksi,ksj)=zwkprojr(jj,ksi,ksj)
             enddo
            enddo
!
!
            do   ksi=1, 3
             do    ksj=1, 3
             zbbwk(ksi,ksj)=(0.0d0,0.0d0)
             do    ksk=1,3
             zbbwk(ksi,ksj)=zbbwk(ksi,ksj)+zlawk(ksi,ksk)*zaawk(ksk,ksj)
             enddo
             enddo
            enddo
!
!

          do   ksi=1, 3
           do    ksj=1, 3
           zaawk(ksi,ksj)=(0.0d0,0.0d0)
            do    ksk=1,3
            zaawk(ksi,ksj)=zaawk(ksi,ksj)+zbbwk(ksi,ksk)*zrawk(ksj,ksk)
              enddo
             enddo
            enddo

!
!
            do    ksi=1, 3                                              
             do    ksj=1, 3
              zopeaf(3*(ii-1)+ksi,3*(jj-1)+ksj)=zaawk(ksi,ksj)
             enddo
            enddo
!
!                                                                       
        enddo                                                           
        enddo                                                           
!                                                                       
!          if(myrank.eq.0) then
!          open(unit=601,file='operas')
!          open(unit=602,file='o-opea')
!          open(unit=603,file='o-opeaf')
!          do  iipp=1, nnf3
!           do   jjpp=1, nnf3
!            write(602,*) iipp,jjpp, cdabs( zopea(iipp,jjpp))
!            write(603,*) iipp,jjpp, cdabs( zopeaf(iipp,jjpp))
!           enddo
!            write(602,*)'   '
!            write(603,*)'   '
!          enddo
!         endif
!
!
!
!
!
!
!                                                                       
!        if(myrank.eq.0) write(*,*)' before zgeev nnf3=',nnf3, 'nf3=',nf3
!                                                                       
!                                                                       
        call zgeev('V', 'V', nnf3, zopeaf, nf3, zrzw, zwkvl,nf3, &          
     &             zwkvr,nf3,zwork, lwork, rwork,info)                  
!                                                                       
!
!         write(*,*)'      '                                             
!         write(*,*)' zgeev  info=',info                                 
!                                                                       
!                                                                       
!   *** check of zgeev ***                                              
!                                                                       
!          aa=0.0d0                                                     
!           do  ii=1, nnf3                                              
!           do  jj=1, nnf3                                              
!            zdd=(0.0d0,0.0d0)                                          
!           do  kk=1, nnf3                                              
!           zdd=zdd+dconjg(zwkvl(kk,ii))*zwkvr(kk,jj)                   
!           enddo                                                       
!           azdd=cdabs(zdd)                                             
!           if(ii.eq.jj)azdd=azdd/azdd-1.0d0                            
!           aa=aa+azdd                                                  
!          enddo                                                        
!         enddo                                                         
!                                                                       
!         write(*,*)' unitary=',aa                                      
!                                                                       
!                                                                       
!                                                                       
!                                                                       
!                                                                       
!         open(unit=12,file='eigenvalue')                                
!         do  ii=1, nnf3                                                 
!         write(12,*) ii,cdabs(zrzw(ii))                                 
!         enddo                                                          
!         close(unit=12)                                                 
!                                                                       
!                                                                       
        rmmx=cdabs(zrzw(1) )                                            
        rmmn=cdabs(zrzw(nnf3))                                          
        rzwa=rmmx*epst                                                  
!                                                                       
!                                                                       
!        open(unit=14,file='eig-proj')                                   
!         write(*,*)' eig min=',rmmn, ' max=',rmmx                       
!         write(*,*)' cutoff=',rzwa                                      
!                                                                       
        kct=0                                                           
          do  kk=1, nnf3                                                
           if(cdabs(zrzw(kk)).gt.rzwa) then                             
!            write(*,*)' zeroclear kk=',kk                               
            kct=kct+1                                                   
           do  ii=1, nnf3                                               
           zwkvl(ii,kk)=(0.0d0,0.0d0)                                   
           enddo                                                        
           endif                                                        
          enddo                                                         
!           write(*,*)' cutoff n=',kct                                   
!!                                                                       
!          goto 9000                                                    
!                                                                       
        return                                                          
 9000   stop                                                            
         end                                                            
!                                                                       
!                                                                       
! * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
      subroutine  sample(zwkvl,zhpf,zwksa01,zwksa02,zwksa03,&
     &                   distel,iawave,ipolep)
! * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
!
!                                                                       
       implicit  real*8(a-h,o-y)                                        
       implicit  complex*16(z)
       implicit  integer(i-n)                                          
!                                                                       
       parameter  (np=500)                                              
       parameter  (nf=np)                                               
       parameter  (nf3=np*3)                                            
       parameter  (ne=2000)                                             
       parameter  (ne3=ne*3)                                            
       parameter  (ns=40000)                                            
       parameter  (nbf=10)
!                                                                       
!                                                                       
!                                                                       
       common    /media/ro,ct,cl,rmu,ray,rtl,rlam                       
       common    /dynam/omega,dka,dkb,eps                               
       common    /fsgrd/xfgrd(nf,3),nnf,nne         
       common    /slgrd/xlgrd(ns,3),dxsl1,dxsl2,dxsl3,nns1,nns2,nns3, &
     &                  ipole   
       common    /fagrd/xfang(nf,2),xcont(3)                            
!                                                                       
       dimension  zgru(3,3,4),xf(3),yf(3)                               
       dimension  xxf(3),yyf(3)                                         
       dimension  zhpf(nf3,3)                                    
       dimension  zwksa01(nf3),zwksa02(nf3),zwksa03(nf3)
       dimension  zwkvl(nf3,nf3)                                   
!                                                                       
!                                                                       
!                                                                       
!       write(*,*)'                     '                             
!       write(*,*)'                     '                               
       nns=nns1*nns2*nns3                                               
       nnf3=nnf*3                                                       
!       write(*,*)' **samp  nns=',nns, ' nnf3=',nnf3                    
       zi=(0.0d0,1.0d0)                                                 
!                                                                       
!         open(unit=701,file='psample')
!                                                                       
!         write(701,*)' sample iawave=',iawave
!                                                                       
         yf(1)=xcont(1)                                                 
         yf(2)=xcont(2)                                                 
         yf(3)=xcont(3)                                                 
!                                                                       
!         write(*,*)xcont(1),xcont(2),xcont(3)

          do  ii=1, nnf                                                 
!                                                                       
          xf(1)=xfgrd(ii,1)                                             
          xf(2)=xfgrd(ii,2)                                             
          xf(3)=xfgrd(ii,3)                                             
!                                                                       
!                                                                       
!
!          write(701,*)' ii=',ii
!          write(701,*)' xf=',xf(1),xf(2),xf(3)
!          write(701,*)' yf=',yf(1),yf(2),yf(3)

           call  greensda(zgru,xf,yf,iawave)          

!
!          write(701,*)'after greensda '
!          write(701,*)' xf=',xf(1),xf(2),xf(3)
!          write(701,*)' yf=',yf(1),yf(2),yf(3)
!
!
!          do  iip=1, 3
!           do  jjp=1, 3
!           write(701,*)iip,jjp,cdabs(zgru(iip,jjp,1))
!          enddo
!          enddo
! 
!         ipolep=kpol
!                                                                       
         do  kk=1,3    
!                                                 
          zhpf(3*(ii-1)+kk,1)=zgru(kk,1,ipolep)                      
          zhpf(3*(ii-1)+kk,2)=zgru(kk,2,ipolep)                     
          zhpf(3*(ii-1)+kk,3)=zgru(kk,3,ipolep)      
!              
         enddo                                                          
       enddo                                                            
!                                                                       
         rn1=(0.0d0,0.0d0)                                              
          do  ii=1, nnf3                                                
           rn1=rn1+zhpf(ii,1)*dconjg(zhpf(ii,1))  &          
     &            +zhpf(ii,2)*dconjg(zhpf(ii,2))  &                     
     &            +zhpf(ii,3)*dconjg(zhpf(ii,3))                        
          enddo                                                         
!                                                                       
          rrn1=1.0d0/dsqrt(rn1)                                         
!                                                                       
!           write(*,*)' sample rrn1=',rrn1
! 
!                                                                       
         do  jj=1, nnf3                                                
          zwksa01(jj)=(0.0d0,0.0d0)                          
          zwksa02(jj)=(0.0d0,0.0d0)                         
          zwksa03(jj)=(0.0d0,0.0d0)      
!                  
           do  kk=1, nnf3                                             
            zwksa01(jj)=zwksa01(jj)+dconjg(zhpf(kk,1))*zwkvl(kk,jj)*rrn1
            zwksa02(jj)=zwksa02(jj)+dconjg(zhpf(kk,2))*zwkvl(kk,jj)*rrn1
            zwksa03(jj)=zwksa03(jj)+dconjg(zhpf(kk,3))*zwkvl(kk,jj)*rrn1
!
           enddo                                                      
          enddo                                                        
!                                                                       
            rn1=ZDOTC(nnf3,zwksa01,1,zwksa01,1)           
            rn2=ZDOTC(nnf3,zwksa02,1,zwksa02,1)          
            rn3=ZDOTC(nnf3,zwksa03,1,zwksa03,1)         
!                                                                       
!                                                                       
            distel=1.0d0/(rn1+rn2+rn3)  
!             write(*,*) ' **samp aa=',distel                       
!                                                                       
!             close(unit=701)
!                                                                       
 8000    continue                                                       
         return                                                         
         end                                                            
!    
!   
!  
